@@ -23,11 +23,11 @@
 #include "dolphinmodel.h"
 #include "dolphincontroller.h"
 #include "dolphinfileitemdelegate.h"
-#include "dolphinsettings.h"
+#include "settings/dolphinsettings.h"
 #include "dolphinsortfilterproxymodel.h"
 #include "dolphinviewautoscroller.h"
 #include "draganddrophelper.h"
-#include "selectionmanager.h"
+#include "viewextensionsfactory.h"
 #include "viewproperties.h"
 #include "zoomlevelinfo.h"
 
@@ -38,7 +38,6 @@
 #include <klocale.h>
 #include <kmenu.h>
 
-#include <QAbstractProxyModel>
 #include <QAction>
 #include <QApplication>
 #include <QHeaderView>
@@ -46,7 +45,9 @@
 #include <QPainter>
 #include <QScrollBar>
 
-DolphinDetailsView::DolphinDetailsView(QWidget* parent, DolphinController* controller) :
+DolphinDetailsView::DolphinDetailsView(QWidget* parent,
+                                       DolphinController* controller,
+                                       DolphinSortFilterProxyModel* proxyModel) :
     QTreeView(parent),
     m_autoResize(true),
     m_expandingTogglePressed(false),
@@ -54,8 +55,9 @@ DolphinDetailsView::DolphinDetailsView(QWidget* parent, DolphinController* contr
     m_useDefaultIndexAt(true),
     m_ignoreScrollTo(false),
     m_controller(controller),
-    m_selectionManager(0),
-    m_autoScroller(0),
+    m_extensionsFactory(0),
+    m_expandableFoldersAction(0),
+    m_expandedUrls(),
     m_font(),
     m_decorationSize(),
     m_band()
@@ -75,9 +77,9 @@ DolphinDetailsView::DolphinDetailsView(QWidget* parent, DolphinController* contr
     setRootIsDecorated(settings->expandableFolders());
     setItemsExpandable(settings->expandableFolders());
     setEditTriggers(QAbstractItemView::NoEditTriggers);
+    setModel(proxyModel);
 
     setMouseTracking(true);
-    m_autoScroller = new DolphinViewAutoScroller(this);
 
     const ViewProperties props(controller->url());
     setSortIndicatorSection(props.sorting());
@@ -88,7 +90,7 @@ DolphinDetailsView::DolphinDetailsView(QWidget* parent, DolphinController* contr
             this, SLOT(synchronizeSortingState(int)));
     headerView->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(headerView, SIGNAL(customContextMenuRequested(const QPoint&)),
-            this, SLOT(configureColumns(const QPoint&)));
+            this, SLOT(configureSettings(const QPoint&)));
     connect(headerView, SIGNAL(sectionResized(int, int, int)),
             this, SLOT(slotHeaderSectionResized(int, int, int)));
     connect(headerView, SIGNAL(sectionHandleDoubleClicked(int)),
@@ -109,14 +111,6 @@ DolphinDetailsView::DolphinDetailsView(QWidget* parent, DolphinController* contr
                 controller, SLOT(triggerItem(const QModelIndex&)));
     }
 
-    if (DolphinSettings::instance().generalSettings()->showSelectionToggle()) {
-        m_selectionManager = new SelectionManager(this);
-        connect(m_selectionManager, SIGNAL(selectionChanged()),
-                this, SLOT(requestActivation()));
-        connect(m_controller, SIGNAL(urlChanged(const KUrl&)),
-                m_selectionManager, SLOT(reset()));
-    }
-
     connect(this, SIGNAL(entered(const QModelIndex&)),
             this, SLOT(slotEntered(const QModelIndex&)));
     connect(this, SIGNAL(viewportEntered()),
@@ -144,32 +138,71 @@ DolphinDetailsView::DolphinDetailsView(QWidget* parent, DolphinController* contr
     connect(view, SIGNAL(showPreviewChanged()),
             this, SLOT(slotShowPreviewChanged()));
 
-    updateDecorationSize(view->showPreview());
 
     setFocus();
     viewport()->installEventFilter(this);
 
-    connect(KGlobalSettings::self(), SIGNAL(kdisplayFontChanged()),
-            this, SLOT(updateFont()));
+    connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)),
+            this, SLOT(slotGlobalSettingsChanged(int)));
 
     m_useDefaultIndexAt = false;
+
+    m_expandableFoldersAction = new QAction(i18nc("@option:check", "Expandable Folders"), this);
+    m_expandableFoldersAction->setCheckable(true);
+    connect(m_expandableFoldersAction, SIGNAL(toggled(bool)),
+            this, SLOT(setFoldersExpandable(bool)));
+
+    connect(this, SIGNAL(expanded(const QModelIndex&)), this, SLOT(slotExpanded(const QModelIndex&)));
+    connect(this, SIGNAL(collapsed(const QModelIndex&)), this, SLOT(slotCollapsed(const QModelIndex&)));
+
+    updateDecorationSize(view->showPreview());
+
+    m_extensionsFactory = new ViewExtensionsFactory(this, controller);
+    m_extensionsFactory->fileItemDelegate()->setMinimizedNameColumn(true);
+    m_extensionsFactory->setAutoFolderExpandingEnabled(settings->expandableFolders());
 }
 
 DolphinDetailsView::~DolphinDetailsView()
 {
 }
 
+QSet<KUrl> DolphinDetailsView::expandedUrls() const
+{
+    return m_expandedUrls;
+}
+
+QRegion DolphinDetailsView::visualRegionForSelection(const QItemSelection& selection) const
+{
+    // We have to make sure that the visualRect of each model index is inside the region.
+    // QTreeView::visualRegionForSelection does not do it right because it assumes implicitly
+    // that all visualRects have the same width, which is in general not the case here.
+    QRegion selectionRegion;
+    const QModelIndexList indexes = selection.indexes();
+
+    foreach(const QModelIndex& index, indexes) {
+        selectionRegion += visualRect(index);
+    }
+
+    return selectionRegion;
+}
+
 bool DolphinDetailsView::event(QEvent* event)
 {
-    if (event->type() == QEvent::Polish) {
-        QHeaderView* headerView = header();
-        headerView->setResizeMode(QHeaderView::Interactive);
-        headerView->setMovable(false);
-
+    switch (event->type()) {
+    case QEvent::Polish:
+        header()->setResizeMode(QHeaderView::Interactive);
         updateColumnVisibility();
+        break;
 
-        hideColumn(DolphinModel::Rating);
-        hideColumn(DolphinModel::Tags);
+    case QEvent::FocusOut:
+        // If a key-press triggers an action that e. g. opens a dialog, the
+        // widget gets no key-release event. Assure that the pressed state
+        // is reset to prevent accidently setting the current index during a selection.
+        m_keyPressed = false;
+        break;
+
+    default:
+        break;
     }
 
     return QTreeView::event(event);
@@ -179,6 +212,7 @@ QStyleOptionViewItem DolphinDetailsView::viewOptions() const
 {
     QStyleOptionViewItem viewOptions = QTreeView::viewOptions();
     viewOptions.font = m_font;
+    viewOptions.fontMetrics = QFontMetrics(m_font);
     viewOptions.showDecorationSelected = true;
     viewOptions.decorationSize = m_decorationSize;
     return viewOptions;
@@ -187,7 +221,11 @@ QStyleOptionViewItem DolphinDetailsView::viewOptions() const
 void DolphinDetailsView::contextMenuEvent(QContextMenuEvent* event)
 {
     QTreeView::contextMenuEvent(event);
-    m_controller->triggerContextMenuRequest(event->pos());
+
+    DetailsModeSettings* settings = DolphinSettings::instance().detailsModeSettings();
+    m_expandableFoldersAction->setChecked(settings->expandableFolders());
+    m_controller->triggerContextMenuRequest(event->pos(),
+                                            QList<QAction*>() << m_expandableFoldersAction);
 }
 
 void DolphinDetailsView::mousePressEvent(QMouseEvent* event)
@@ -197,20 +235,14 @@ void DolphinDetailsView::mousePressEvent(QMouseEvent* event)
     const QModelIndex current = currentIndex();
     QTreeView::mousePressEvent(event);
 
-    m_expandingTogglePressed = false;
+    m_expandingTogglePressed = isAboveExpandingToggle(event->pos());
+
     const QModelIndex index = indexAt(event->pos());
     const bool updateState = index.isValid() &&
                              (index.column() == DolphinModel::Name) &&
                              (event->button() == Qt::LeftButton);
     if (updateState) {
-        // TODO: See comment in DolphinIconsView::mousePressEvent(). Only update
-        // the state if no expanding/collapsing area has been hit:
-        const QRect rect = visualRect(index);
-        if (event->pos().x() >= rect.x() + indentation()) {
-            setState(QAbstractItemView::DraggingState);
-        } else {
-            m_expandingTogglePressed = true;
-        }
+        setState(QAbstractItemView::DraggingState);
     }
 
     if (!index.isValid() || (index.column() != DolphinModel::Name)) {
@@ -219,8 +251,8 @@ void DolphinDetailsView::mousePressEvent(QMouseEvent* event)
             m_controller->replaceUrlByClipboard();
         }
 
-        const Qt::KeyboardModifiers modifier = QApplication::keyboardModifiers();
-        if (!(modifier & Qt::ShiftModifier) && !(modifier & Qt::ControlModifier)) {
+        const Qt::KeyboardModifiers mod = QApplication::keyboardModifiers();
+        if (!m_expandingTogglePressed && !(mod & Qt::ShiftModifier) && !(mod & Qt::ControlModifier)) {
             clearSelection();
         }
 
@@ -246,6 +278,14 @@ void DolphinDetailsView::mousePressEvent(QMouseEvent* event)
 
 void DolphinDetailsView::mouseMoveEvent(QMouseEvent* event)
 {
+    if (m_expandingTogglePressed) {
+        // Per default QTreeView starts either a selection or a drag operation when dragging
+        // the expanding toggle button (Qt-issue - see TODO comment in DolphinIconsView::mousePressEvent()).
+        // Turn off this behavior in Dolphin to stay predictable:
+        setState(QAbstractItemView::NoState);
+        return;
+    }
+
     if (m_band.show) {
         const QPoint mousePos = event->pos();
         const QModelIndex index = indexAt(mousePos);
@@ -267,31 +307,25 @@ void DolphinDetailsView::mouseMoveEvent(QMouseEvent* event)
         // QTreeView::mouseMoveEvent(event);
         QAbstractItemView::mouseMoveEvent(event);
     }
-
-    if (m_expandingTogglePressed) {
-        // Per default QTreeView starts either a selection or a drag operation when dragging
-        // the expanding toggle button (Qt-issue - see TODO comment in DolphinIconsView::mousePressEvent()).
-        // Turn off this behavior in Dolphin to stay predictable:
-        clearSelection();
-        setState(QAbstractItemView::NoState);
-    }
 }
 
 void DolphinDetailsView::mouseReleaseEvent(QMouseEvent* event)
 {
-    const QModelIndex index = indexAt(event->pos());
-    if (index.isValid() && (index.column() == DolphinModel::Name)) {
-        QTreeView::mouseReleaseEvent(event);
-    } else {
-        // don't change the current index if the cursor is released
-        // above any other column than the name column, as the other
-        // columns act as viewport
-        const QModelIndex current = currentIndex();
-        QTreeView::mouseReleaseEvent(event);
-        selectionModel()->setCurrentIndex(current, QItemSelectionModel::Current);
+    if (!m_expandingTogglePressed) {
+        const QModelIndex index = indexAt(event->pos());
+        if (index.isValid() && (index.column() == DolphinModel::Name)) {
+            QTreeView::mouseReleaseEvent(event);
+        } else {
+            // don't change the current index if the cursor is released
+            // above any other column than the name column, as the other
+            // columns act as viewport
+            const QModelIndex current = currentIndex();
+            QTreeView::mouseReleaseEvent(event);
+            selectionModel()->setCurrentIndex(current, QItemSelectionModel::Current);
+        }
     }
-
     m_expandingTogglePressed = false;
+
     if (m_band.show) {
         setState(NoState);
         updateElasticBand();
@@ -404,18 +438,7 @@ void DolphinDetailsView::resizeEvent(QResizeEvent* event)
 
 void DolphinDetailsView::wheelEvent(QWheelEvent* event)
 {
-    if (m_selectionManager != 0) {
-        m_selectionManager->reset();
-    }
-
-    // let Ctrl+wheel events propagate to the DolphinView for icon zooming
-    if (event->modifiers() & Qt::ControlModifier) {
-        event->ignore();
-        return;
-    }
-
-    const int height = m_decorationSize.height();
-    const int step = (height >= KIconLoader::SizeHuge) ? height / 10 : (KIconLoader::SizeHuge - height) / 2;
+    const int step = m_decorationSize.height();
     verticalScrollBar()->setSingleStep(step);
     QTreeView::wheelEvent(event);
 }
@@ -423,15 +446,17 @@ void DolphinDetailsView::wheelEvent(QWheelEvent* event)
 void DolphinDetailsView::currentChanged(const QModelIndex& current, const QModelIndex& previous)
 {
     QTreeView::currentChanged(current, previous);
-    if (current.isValid() && !m_autoScroller->isActive()) {
-        scrollTo(current);
-    }
+    m_extensionsFactory->handleCurrentIndexChange(current, previous);
 
     // Stay consistent with QListView: When changing the current index by key presses,
     // also change the selection.
     if (m_keyPressed) {
-        selectionModel()->select(current, QItemSelectionModel::ClearAndSelect);
+        setCurrentIndex(current);
     }
+
+    // If folders are expanded, the width which is available for editing may have changed
+    // because it depends on the level of the current item in the folder hierarchy.
+    adjustMaximumSizeForEditing(current);
 }
 
 bool DolphinDetailsView::eventFilter(QObject* watched, QEvent* event)
@@ -451,10 +476,22 @@ QModelIndex DolphinDetailsView::indexAt(const QPoint& point) const
     // the blank portion of the name column counts as empty space
     const QModelIndex index = QTreeView::indexAt(point);
     const bool isAboveEmptySpace  = !m_useDefaultIndexAt &&
-                                    (index.column() == KDirModel::Name) && !nameColumnRect(index).contains(point);
+                                    (index.column() == KDirModel::Name) && !visualRect(index).contains(point);
     return isAboveEmptySpace ? QModelIndex() : index;
 }
 
+QRect DolphinDetailsView::visualRect(const QModelIndex& index) const
+{
+    QRect rect = QTreeView::visualRect(index);
+    const KFileItem item = m_controller->itemForIndex(index);
+    if (!item.isNull()) {
+        const int width = DolphinFileItemDelegate::nameColumnWidth(item.text(), viewOptions());
+        rect.setWidth(width);
+    }
+
+    return rect;
+}
+
 void DolphinDetailsView::setSelection(const QRect& rect, QItemSelectionModel::SelectionFlags command)
 {
     // We must override setSelection() as Qt calls it internally and when this happens
@@ -548,27 +585,30 @@ void DolphinDetailsView::setZoomLevel(int level)
     updateDecorationSize(showPreview);
 }
 
-
 void DolphinDetailsView::slotShowPreviewChanged()
 {
     const DolphinView* view = m_controller->dolphinView();
     updateDecorationSize(view->showPreview());
 }
 
-void DolphinDetailsView::configureColumns(const QPoint& pos)
+void DolphinDetailsView::configureSettings(const QPoint& pos)
 {
     KMenu popup(this);
     popup.addTitle(i18nc("@title:menu", "Columns"));
 
+    // add checkbox items for each column
     QHeaderView* headerView = header();
-    for (int i = DolphinModel::Size; i <= DolphinModel::Type; ++i) {
+    const int columns = model()->columnCount();
+    for (int i = 0; i < columns; ++i) {
         const int logicalIndex = headerView->logicalIndex(i);
-        const QString text = model()->headerData(i, Qt::Horizontal).toString();
+        const QString text = model()->headerData(logicalIndex, Qt::Horizontal).toString();
         QAction* action = popup.addAction(text);
         action->setCheckable(true);
         action->setChecked(!headerView->isSectionHidden(logicalIndex));
-        action->setData(i);
+        action->setData(logicalIndex);
+        action->setEnabled(logicalIndex != DolphinModel::Name);
     }
+    popup.addSeparator();
 
     QAction* activatedAction = popup.exec(header()->mapToGlobal(pos));
     if (activatedAction != 0) {
@@ -594,16 +634,41 @@ void DolphinDetailsView::configureColumns(const QPoint& pos)
 
 void DolphinDetailsView::updateColumnVisibility()
 {
+    QHeaderView* headerView = header();
+    disconnect(headerView, SIGNAL(sectionMoved(int, int, int)),
+               this, SLOT(saveColumnPositions()));
+
+    const DetailsModeSettings* settings = DolphinSettings::instance().detailsModeSettings();
+    const QList<int> columnPositions = settings->columnPositions();
+    
     const KFileItemDelegate::InformationList list = m_controller->dolphinView()->additionalInfo();
-    for (int i = DolphinModel::Size; i <= DolphinModel::Type; ++i) {
+    for (int i = DolphinModel::Name; i <= DolphinModel::Version; ++i) {
         const KFileItemDelegate::Information info = infoForColumn(i);
-        const bool hide = !list.contains(info);
+        const bool hide = !list.contains(info) && (i != DolphinModel::Name);
         if (isColumnHidden(i) != hide) {
             setColumnHidden(i, hide);
         }
+        
+        const int from = headerView->visualIndex(i);
+        headerView->moveSection(from, columnPositions[i]);
+    }
+    
+    resizeColumns();
+
+    connect(headerView, SIGNAL(sectionMoved(int, int, int)),
+            this, SLOT(saveColumnPositions()));
+
+}
+
+void DolphinDetailsView::saveColumnPositions()
+{
+    QList<int> columnPositions;
+    for (int i = DolphinModel::Name; i <= DolphinModel::Version; ++i) {
+        columnPositions.append(header()->visualIndex(i));
     }
 
-    resizeColumns();
+    DetailsModeSettings* settings = DolphinSettings::instance().detailsModeSettings();
+    settings->setColumnPositions(columnPositions);
 }
 
 void DolphinDetailsView::slotHeaderSectionResized(int logicalIndex, int oldSize, int newSize)
@@ -619,6 +684,8 @@ void DolphinDetailsView::slotHeaderSectionResized(int logicalIndex, int oldSize,
     if ((QApplication::mouseButtons() & Qt::LeftButton) && header()->underMouse()) {
         disableAutoResizing();
     }
+
+    adjustMaximumSizeForEditing(currentIndex());
 }
 
 void DolphinDetailsView::slotActivationChanged(bool active)
@@ -636,14 +703,23 @@ void DolphinDetailsView::requestActivation()
     m_controller->requestActivation();
 }
 
-void DolphinDetailsView::updateFont()
+void DolphinDetailsView::slotGlobalSettingsChanged(int category)
 {
+    Q_UNUSED(category);
+
     const DetailsModeSettings* settings = DolphinSettings::instance().detailsModeSettings();
     Q_ASSERT(settings != 0);
-
     if (settings->useSystemFont()) {
         m_font = KGlobalSettings::generalFont();
     }
+    //Disconnect then reconnect, since the settings have been changed, the connection requirements may have also.
+    disconnect(this, SIGNAL(clicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    disconnect(this, SIGNAL(doubleClicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    if (KGlobalSettings::singleClick()) {
+        connect(this, SIGNAL(clicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    } else {
+        connect(this, SIGNAL(doubleClicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    }
 }
 
 void DolphinDetailsView::updateElasticBandSelection()
@@ -707,8 +783,7 @@ void DolphinDetailsView::updateElasticBandSelection()
                 return;
             }
         }
-    }
-    else {
+    } else {
         // This is the only piece of optimization data that needs to be explicitly
         // discarded.
         m_band.lastSelectionOrigin = QPoint();
@@ -760,7 +835,7 @@ void DolphinDetailsView::updateElasticBandSelection()
    QModelIndex toggleIndexRangeBegin = QModelIndex();
 
    do {
-       QRect currIndexRect = nameColumnRect(currIndex);
+       QRect currIndexRect = visualRect(currIndex);
 
         // Update some optimization info as we go.
        const int cr = currIndexRect.right();
@@ -822,6 +897,7 @@ void DolphinDetailsView::updateElasticBandSelection()
        currIndex = nextIndex;
     } while (!allItemsInBoundDone);
 
+
     selectionModel()->select(itemsToToggle, QItemSelectionModel::Toggle);
 
     m_band.lastSelectionOrigin = m_band.origin;
@@ -829,6 +905,63 @@ void DolphinDetailsView::updateElasticBandSelection()
     m_band.ignoreOldInfo = false;
 }
 
+void DolphinDetailsView::setFoldersExpandable(bool expandable)
+{
+    if (!expandable) {
+        // collapse all expanded folders, as QTreeView::setItemsExpandable(false)
+        // does not do this task
+        const int rowCount = model()->rowCount();
+        for (int row = 0; row < rowCount; ++row) {
+            setExpanded(model()->index(row, 0), false);
+        }
+    }
+    DetailsModeSettings* settings = DolphinSettings::instance().detailsModeSettings();
+    settings->setExpandableFolders(expandable);
+    setRootIsDecorated(expandable);
+    setItemsExpandable(expandable);
+
+    // The width of the space which is available for editing has changed
+    // because of the (dis)appearance of the expanding toggles
+    adjustMaximumSizeForEditing(currentIndex());
+}
+
+void DolphinDetailsView::slotExpanded(const QModelIndex& index)
+{
+    KFileItem item = m_controller->itemForIndex(index);
+    if (!item.isNull()) {
+        m_expandedUrls.insert(item.url());
+    }
+}
+
+void DolphinDetailsView::slotCollapsed(const QModelIndex& index)
+{
+    KFileItem item = m_controller->itemForIndex(index);
+    if (!item.isNull()) {
+        m_expandedUrls.remove(item.url());
+    }
+}
+
+void DolphinDetailsView::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
+{
+    removeExpandedIndexes(parent, start, end);
+    QTreeView::rowsAboutToBeRemoved(parent, start, end);
+}
+
+void DolphinDetailsView::removeExpandedIndexes(const QModelIndex& parent, int start, int end)
+{
+    if (m_expandedUrls.isEmpty()) {
+        return;
+    }
+
+    for (int row = start; row <= end; row++) {
+        const QModelIndex index = model()->index(row, 0, parent);
+        if (isExpanded(index)) {
+            slotCollapsed(index);
+            removeExpandedIndexes(index, 0, model()->rowCount(index) - 1);
+        }
+    }
+}
+
 void DolphinDetailsView::updateDecorationSize(bool showPreview)
 {
     DetailsModeSettings* settings = DolphinSettings::instance().detailsModeSettings();
@@ -836,10 +969,6 @@ void DolphinDetailsView::updateDecorationSize(bool showPreview)
     setIconSize(QSize(iconSize, iconSize));
     m_decorationSize = QSize(iconSize, iconSize);
 
-    if (m_selectionManager != 0) {
-        m_selectionManager->reset();
-    }
-
     doItemsLayout();
 }
 
@@ -871,13 +1000,14 @@ void DolphinDetailsView::resizeColumns()
     QHeaderView* headerView = header();
     QFontMetrics fontMetrics(viewport()->font());
 
-    int columnWidth[KDirModel::ColumnCount];
-    columnWidth[KDirModel::Size] = fontMetrics.width("00000 Items");
-    columnWidth[KDirModel::ModifiedTime] = fontMetrics.width("0000-00-00 00:00");
-    columnWidth[KDirModel::Permissions] = fontMetrics.width("xxxxxxxxxx");
-    columnWidth[KDirModel::Owner] = fontMetrics.width("xxxxxxxxxx");
-    columnWidth[KDirModel::Group] = fontMetrics.width("xxxxxxxxxx");
-    columnWidth[KDirModel::Type] = fontMetrics.width("XXXX Xxxxxxx");
+    int columnWidth[DolphinModel::Version + 1];
+    columnWidth[DolphinModel::Size] = fontMetrics.width("00000 Items");
+    columnWidth[DolphinModel::ModifiedTime] = fontMetrics.width("0000-00-00 00:00");
+    columnWidth[DolphinModel::Permissions] = fontMetrics.width("xxxxxxxxxx");
+    columnWidth[DolphinModel::Owner] = fontMetrics.width("xxxxxxxxxx");
+    columnWidth[DolphinModel::Group] = fontMetrics.width("xxxxxxxxxx");
+    columnWidth[DolphinModel::Type] = fontMetrics.width("XXXX Xxxxxxx");
+    columnWidth[DolphinModel::Version] = fontMetrics.width("xxxxxxxx");
 
     int requiredWidth = 0;
     for (int i = KDirModel::Size; i <= KDirModel::Type; ++i) {
@@ -913,16 +1043,38 @@ void DolphinDetailsView::resizeColumns()
     headerView->resizeSection(KDirModel::Name, columnWidth[KDirModel::Name]);
 }
 
-QRect DolphinDetailsView::nameColumnRect(const QModelIndex& index) const
+bool DolphinDetailsView::isAboveExpandingToggle(const QPoint& pos) const
 {
-    QRect rect = visualRect(index);
-    const KFileItem item = m_controller->itemForIndex(index);
-    if (!item.isNull()) {
-        const int width = DolphinFileItemDelegate::nameColumnWidth(item.text(), viewOptions());
-        rect.setWidth(width);
-    }
+    // QTreeView offers no public API to get the information whether an index has an
+    // expanding toggle and what boundaries the toggle has. The following approach
+    // also assumes a toggle for file items.
+    if (itemsExpandable()) {
+        const QModelIndex index = QTreeView::indexAt(pos);
+        if (index.isValid() && (index.column() == KDirModel::Name)) {
+            QRect rect = visualRect(index);
+            const int toggleSize = rect.height();
+            if (isRightToLeft()) {
+                rect.moveRight(rect.right());
+            } else {
+                rect.moveLeft(rect.x() - toggleSize);
+            }
+            rect.setWidth(toggleSize);
 
-    return rect;
+            QStyleOption opt;
+            opt.initFrom(this);
+            opt.rect = rect;
+            rect = style()->subElementRect(QStyle::SE_TreeViewDisclosureItem, &opt, this);
+
+            return rect.contains(pos);
+        }
+    }
+    return false;
+}
+
+void DolphinDetailsView::adjustMaximumSizeForEditing(const QModelIndex& index)
+{
+    // Make sure that the full width of the "Name" column is available for "Rename Inline"
+    m_extensionsFactory->fileItemDelegate()->setMaximumSize(QTreeView::visualRect(index).size());
 }
 
 DolphinDetailsView::ElasticBand::ElasticBand() :
